import { ContainerOptions, PropertyOptions, SerdeOptions } from './option'
import { Nullable } from './utils'

const ContextKind = Object.freeze({
  Class: 'class',
  Method: 'method',
  Getter: 'getter',
  Setter: 'setter',
  Field: 'field',
  Accessor: 'accessor'
} as const)

function decorateClass(target: Function, context: DecoratorContext, options: ContainerOptions) {
  const serde = context.metadata.serde as Nullable<SerdeOptions>

  context.metadata.serde = new SerdeOptions(
    target,
    options,
    serde?.properties
  )
}

function decorateField(context: DecoratorContext, options: PropertyOptions) {
  const serde = (context.metadata.serde as SerdeOptions) || new SerdeOptions(undefined)
  serde.properties.set(context.name as string, options)
  context.metadata.serde = serde
}

export function serde(options: ContainerOptions | PropertyOptions) {
  return function(target: any, context: DecoratorContext) {
    switch (context.kind) {
      case ContextKind.Class:
        decorateClass(target, context, options)
        break

      case ContextKind.Field:
        decorateField(context, options)

      default:
        break
    }
  }
}

